@@ -28,8 +28,12 @@ sub import {
     my ($class) = @_;
 
     # If we are building or in test here, just exit
-    # as travis build servers will not have Tk installed
-    return if $ENV{AUTHOR_TESTING} || $ENV{RELEASE_TESTING};
+    # as the build servers will not have Tk installed
+    if ( $ENV{AUTHOR_TESTING} || $ENV{RELEASE_TESTING} ) {
+        print STDERR
+            "skipping initialisation; AUTHOR_TESTING or RELEASE_TESTING are set\n";
+        return;
+    }
 
     # Find what windows module we should be using and just overlay it into
     # this object
